import { useLoaderData, useActionData, Form } from "@remix-run/react";
import { unstable_createFileUploadHandler, unstable_parseMultipartFormData } from "@remix-run/node";
import prisma from "~/utils/db.server";
import { redirect } from "@remix-run/node";
import { Link } from "@remix-run/react";
import Overlay from "~/components/Overlay";
import { json } from "@remix-run/node";

export async function action({ request }) {
  const clonedData = request.clone();
  const formData = await clonedData.formData();
  const title = formData.get("title");
  const post = formData.get("post");

  const fileUploadHandler = unstable_createFileUploadHandler({
    directory: './public/uploads/',
    maxPartSize: 500000,
    file: ({ filename }) => filename,
  });

  const errors = {};
  let multiPartformdata;
  let imageName;
  try {
    multiPartformdata = await unstable_parseMultipartFormData(request, fileUploadHandler);
    imageName = multiPartformdata.get("image").name;
  } catch (err) {
    errors.image = "Image size too big";
  }
  
  if (typeof title !== "string" || title.length > 50 || title.length < 3) {
    errors.title = "Title too long or short";
  };
  if (typeof post !== "string" || post.length > 50 || post.length < 3) {
    errors.post = "Post too long or short";
  };
  if (Object.keys(errors).length) {
    return json(errors, { status: 422 });
  }

  const newThread = await prisma.thread.create({
    data: {
      title: title,
      post: post,
      imageName: imageName,
    },
  });

  return redirect(`threads/${newThread.id}`);
}

export async function loader() {
  const data = await prisma.thread.findMany({
    include: {
      posts: true,
    },
  });

  return data;
}

export default function Index() {
  const data = useLoaderData();
  const errors = useActionData();

  return (
    <Overlay>
      <div className="bg-ctp-crust outline rounded-xl shadow-lg shadow-ctp-red max-w-2xl mx-auto p-4 space-y-4">
        <h1 className="text-center text-4xl font-semibold text-transparent bg-clip-text bg-gradient-to-br from-ctp-subtext0 to-ctp-subtext1 ">Create a thread</h1>
        <Form className="rounded flex flex-wrap text-center justify-center items-center " method="post" encType="multipart/form-data">
            <label className="flex flex-col m-2" htmlFor="title-input"><span className="font-semibold text-ctp-subtext0">Title</span>
              <textarea className=" bg-ctp-surface0 m-1 p-1 rounded-md shadow shadow-ctp-overlay0" required minLength={3} type="text" name="title" placeholder="Lain" />
              <p>{errors ? errors.title : ''}</p>
            </label>
            <label className="flex flex-col m-2" htmlFor="post-input"><span className="font-semibold text-ctp-subtext0">Post</span> 
              <textarea className=" bg-ctp-surface0 m-1 p-1 rounded shadow shadow-ctp-overlay0" required minLength={3} type="text" name="post" placeholder="Iwakura" />
              <p>{errors ? errors.post : ''}</p>
            </label>
            <label className="flex flex-col m-2" htmlFor="image-upload"><span className="font-semibold text-ctp-subtext0">Image</span>
              <input className="block w-full px-3 py-1.5 text-base font-semibold text-ctp-text bg-ctp-surface0 bg-clip-padding rounded transition ease-in-out m-0" required accept="image/*" type="file" id="image-upload" name="image" />
              <p>{errors ? errors.image : ''}</p>
            </label>
            <br />
            <button className=" bg-ctp-surface1 shadow shadow-ctp-overlay0 hover hover:bg-ctp-surface2 py-2 px-4 hover:underline rounded-full" type="submit">Create!</button>   
        </Form>
      </div>
      {data.length > 0 ?
        <ul className="flex flex-wrap justify-around items-center p-8">
          {data.map(thread => 
            <li className="bg-ctp-mantle shadow-xl outline shadow-ctp-pink rounded-xl p-4 m-4 hover:shadow-xl scale-100 hover:scale-110 duration-500" key={thread.id}>
              <Link className="space-y-4 flex flex-col" to={`threads/${thread.id}`}>
                <div className="flex items-center justify-center">
                  <img className="w-48 rounded-xl " src={`/uploads/${thread.imageName}`} alt="" />
                </div>
                <div>
                  <h2 className="text-2xl font-semibold text-center tracking-tighter ">{thread.title}</h2>
                  <p className=" text-center text-ctp-subtext0">{thread.post}</p>
                  <p className=" text-xs text-center text-ctp-subtext1 ">Posts: {thread.posts.length}</p>
                </div>
              </Link>
            </li>  
          )}
        </ul>
        :
        <h3 className="flex p-20 justify-center items-center text-center text-2xl">no threads yet</h3>
      }
    </Overlay>
  );
};
